"""This module is a collection of old functions.

    * unpack_txt_512 - unpacks the 'txt' data files with 512 timestamps
    per acquisition cycle
    * unpack_txt_10 - unpacks the 'txt' data files with 10 timestamps per
    acquisition cycle
    * unpack_binary_10 - unpacks the 'dat' data files with 10 timestamps
    per acquisition cycle
    * unpack_binary_512 - unpack the 'dat' data files with 512 timestamps
    per acquisition point
    * unpack_binary_flex - unpacks the 'dat' data files with a given number of
    timestamps per acquisition cycle

"""

from struct import unpack
import numpy as np
import sys
import os
from tools.calibrate import calibrate_load


def unpack_binary_flex(filename, timestamps: int = 512):
    """Unpacks the 'dat' data files with certain timestamps per acquistion
    cycle.

    Parameters
    ----------
    filename : str
        File with data from LinoSPAD2 in which precisely timestamps lines
        of data per acquistion cycle is written.
    timestamps: int, optional
        Number of binary-encoded timestamps in the 'dat' file. The default
        value is 512.

    Returns
    -------
    data_matrix : array_like
        A 2D matrix (256 pixels by timestamps X number-of-cycles) of
        timestamps.

    """

    timestamp_list = []
    address_list = []

    with open(filename, "rb") as f:
        while True:
            rawpacket = f.read(4)  # read 32 bits
            if not rawpacket:
                break  # stop when the are no further 4 bytes to readout
            packet = unpack("<I", rawpacket)
            if (packet[0] >> 31) == 1:  # check validity bit: if 1
                # - timestamp is valid
                timestamp = packet[0] & 0xFFFFFFF  # cut the higher bits,
                # leave only timestamp ones
                # 2.5 ns from TDC 400 MHz clock read out 140 bins from 35
                # elements of the delay line - average bin size is 17.857 ps
                timestamp = timestamp * 17.857  # in ps
            else:
                timestamp = -1
            timestamp_list.append(timestamp)
            address = (packet[0] >> 28) & 0x3  # gives away only zeroes -
            # not in this firmware??
            address_list.append(address)
    # rows=#pixels, cols=#cycles
    data_matrix = np.zeros((256, int(len(timestamp_list) / 256)))

    noc = len(timestamp_list) / timestamps / 256  # number of cycles,
    # timestamps data lines per pixel per cycle, 256 pixels

    # pack the data from a 1D array into a 2D matrix
    k = 0
    while k != noc:
        i = 0
        while i < 256:
            data_matrix[i][
                k * timestamps : k * timestamps + timestamps
            ] = timestamp_list[
                (i + 256 * k) * timestamps : (i + 256 * k) * timestamps + timestamps
            ]
            i = i + 1
        k = k + 1
    return data_matrix


def unpack_numpy(filename, timestamps):
    rawFile = np.fromfile(filename, dtype=np.uint32)  # read data
    data = (rawFile & 0xFFFFFFF).astype(int) * 17.857  # Multiply with the lowes bin
    data[np.where(rawFile < 0x80000000)] = -1  # Mask not valid data
    nmrCycles = int(len(data) / timestamps / 256)  # number of cycles,
    data_matrix = (
        data.reshape((timestamps, nmrCycles * 256), order="F")
        .reshape((timestamps, 256, -1), order="F")
        .transpose((0, 2, 1))
        .reshape((-1, 256), order="F")
        .transpose()
    )  # reshape the matrix
    return data_matrix


def unpack_calib(filename, board_number: str, timestamps: int = 512):
    """
    Function for unpacking the .dat data files using the calibration
    data. The output is a matrix of '256 x timestamps*number_of_cycles'
    timestamps in ps.

    Parameters
    ----------
    filename : str
        Name of the .dat file.
    board_number : str
        LinoSPAD2 board number.
    timestamps : int, optional
        Number of timestamps per acquisition cycle per pixel. The default is 512.

    Returns
    -------
    data_matrix : ndarray
        Matrix of '256 x timestamps*number_of_cycles' timestamps.

    """

    # read data by 32 bit words
    rawFile = np.fromfile(filename, dtype=np.uint32)
    # lowest 28 bits are the timestamp; convert to longlong, int is not enough
    data = (rawFile & 0xFFFFFFF).astype(np.longlong)
    # mask nonvalid data with '-1'
    data[np.where(rawFile < 0x80000000)] = -1
    # number of acquisition cycles
    cycles = int(len(data) / timestamps / 256)

    data_matrix = (
        data.reshape(cycles, 256, timestamps)
        .transpose((1, 0, 2))
        .reshape(256, timestamps * cycles)
    )
    # path to the current script, two levels up (the script itself is in the path) and
    # one level down to the calibration data
    path_calib_data = os.path.realpath(__file__) + "/../.." + "/calibration_data"

    try:
        cal_mat = calibrate_load(path_calib_data, board_number)
    except FileNotFoundError:
        print(
            "No .csv file with the calibration data was found, check the path "
            "or run the calibration."
        )
        sys.exit()
    for i in range(256):
        ind = np.where(data_matrix[i] >= 0)[0]
        data_matrix[i, ind] = (
            data_matrix[i, ind] - data_matrix[i, ind] % 140
        ) * 17.857 + cal_mat[i, (data_matrix[i, ind] % 140)]
    return data_matrix


def unpack_2212(filename, board_number, fw_ver, timestamps: int = 512):
    """Unpack binary data from LinoSPAD2 2212 firmware version.

    Function for unpacking data into a dictionary for the firmware versions 2212 "skip"
    and "block". Uses the calibration data.

    Parameters
    ----------
    filename : str
        The name of the ".dat" data file.
    board_number : str
        The LinoSPAD2 dautherboard number.
    fw_ver : str
        2212 firmware version: either "skip" or "block".
    timestamps : int, optional
        Number of timestamps per acquisition cycle per pixel. The default is 512.

    Returns
    -------
    dict
        Matrix of timestamps with 256 rows. Output is a dictionary as the number
        of columns is different for each row.

    """
    timestamp_list = {}

    for i in range(0, 256):
        timestamp_list["{}".format(i)] = []

    # Variables that follow the cycle and the TDC numbers
    cycler = 0
    tdc = 0

    # Function for assigning pixel addresses bases on the type of the 2212
    # firmware version
    def _pix_num(tdc_num, pix_coor):
        if fw_ver == "2212b":
            out = 4 * tdc_num + pix_coor
        elif fw_ver == "2212s":
            out = tdc_num + 64 * pix_coor

        return out

    with open(filename, "rb") as f:
        while True:
            rawpacket = f.read(4)
            # All steps are in units of 32 bits
            # Reaching the end of a cycle, assign a '-1' to each pixel
            if not cycler % (32 * 65 * timestamps) and cycler != 0:
                for i in range(256):
                    timestamp_list["{}".format(i)].append(-1)
            # Next TDC
            if not cycler % (32 * timestamps) and cycler != 0:
                tdc += 1
            cycler += 32
            # Cut the 64th (TDC=[0,...63]) that does not contain timestamps
            if tdc != 0 and tdc == 64:
                continue
            # Reset the TDC number - end of cycle
            if tdc != 0 and tdc == 65:
                tdc = 0
            if not rawpacket:
                break
            packet = unpack("<I", rawpacket)
            if (packet[0] >> 31) == 1:
                pix_coor = (packet[0] >> 28) & 0x3
                address = _pix_num(tdc, pix_coor)
                timestamp_list["{}".format(address)].append((packet[0] & 0xFFFFFFF))

    for key in timestamp_list:
        timestamp_list[key] = np.array(timestamp_list[key]).astype(np.longlong)

    # path to the current script, two levels up (the script itself is in the path) and
    # one level down to the calibration data
    path_calib_data = os.path.realpath(__file__) + "/../.." + "/calibration_data"

    try:
        cal_mat = calibrate_load(path_calib_data, board_number)
    except FileNotFoundError:
        print(
            "No .csv file with the calibration data was found, check the path "
            "or run the calibration."
        )
        sys.exit()
    for i in range(256):
        ind = np.where(np.array(timestamp_list["{}".format(i)]) >= 0)[0]
        if not np.any(ind):
            continue
        timestamp_list["{}".format(i)][ind] = (
            timestamp_list["{}".format(i)][ind]
            - timestamp_list["{}".format(i)][ind] % 140
        ) * 17.857 + cal_mat[i, (timestamp_list["{}".format(i)][ind] % 140)]

    return timestamp_list



"""Module for handling calibration data.

The following functions are provided:

    * calibrate_save - unpack the calibration data and save the output
    calibration matrix into a '.csv' file

    * calibrate_load - loads the calibration matrix from a '.csv' file

"""
import numpy as np
import os
import glob
import pandas as pd


def calibrate_save(path, timestamps: int = 512):
    """
    Function for calculating the calibration matrix and saving it into a .csv
    file. The data file used for the calculation should be taken with the
    sensor uniformly illuminated by ambient light.

    Parameters
    ----------
    path : str
        Path to the data file.
    timestamps : int, optional
        Number of timestamps per acquisition cycle per pixel. The default is 512.

    Returns
    -------
    None.

    """

    os.chdir(path)
    filename = glob.glob("*.dat*")[0]

    # read data by 32 bit words
    rawFile = np.fromfile(filename, dtype=np.uint32)
    # lowest 28 bits are the timestamp; convert to ps
    data = (rawFile & 0xFFFFFFF).astype(int) % 140
    # mask nonvalid data with '-1'; 0x80000000 - the 31st, validity bin
    data[np.where(rawFile < 0x80000000)] = -1
    # number of acquisition cycles
    cycles = int(len(data) / timestamps / 256)

    data_matrix = (
        data.reshape(cycles, 256, timestamps)
        .transpose((1, 0, 2))
        .reshape(256, timestamps * cycles)
    )

    # calibration matrix
    cal_mat = np.zeros((256, 140))
    bins = np.arange(0, 141, 1)

    for i in range(256):
        # sort the data into 140 bins
        counts, bin_edges = np.histogram(data_matrix[i], bins=bins)
        # redefine the bin edges using the bin population from above
        cal_mat[i] = np.cumsum(counts) / np.cumsum(counts).max() * 2500
    cal_mat_df = pd.DataFrame(cal_mat)
    cal_mat_df.to_csv("Calibration_data.csv")


def calibrate_load(path, board_number: str):
    """
    Function for loading the calibration data.

    Parameters
    ----------
    path : str
        Path to the .csv file with the calibration matrix.
    board_number: str
        The LinoSPAD2 board number.

    Returns
    -------
    data_matrix : ndarray
        Matrix of 256x140 with the calibrated data.

    """

    path_to_back = os.getcwd()
    os.chdir(path)

    file = glob.glob("*{}*".format(board_number))[0]

    data_matrix = np.genfromtxt(file, delimiter=",", skip_header=1)
    data_matrix = np.delete(data_matrix, 0, axis=1)

    os.chdir(path_to_back)

    return data_matrix



from PyQt5.QtWidgets import QWidget, QVBoxLayout
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
from tools.calc_diff import calc_diff as cd
import os
from glob import glob
from tools.unpack_data import unpack_calib
import numpy as np
import matplotlib.pyplot as plt


class Delta_tCanvas(QWidget):
    def __init__(self, parent=None, width=7, height=4, dpi=100):
        super(Delta_tCanvas, self).__init__(parent)
        # a figure instance to plot on
        self.figure = Figure(figsize=(width, height), dpi=dpi)
        self.canvas = FigureCanvas(self.figure)
        self.toolbar = NavigationToolbar(self.canvas, self)
        self.axes = self.figure.add_subplot(111)
        self.figure.subplots_adjust(left=0.15, right=0.97, top=0.96, bottom=0.17)

        # creating a Vertical Box layout
        self.layout = QVBoxLayout(self)
        self.layout.addWidget(self.canvas)
        self.layout.addWidget(self.toolbar)

        self.setLayout(self.layout)

        self._setplotparameters()

    def _setplotparameters(self):
        plt.rcParams.update({"font.size": 12})
        self.axes.set_xlabel("\u0394t [ps]")
        self.axes.set_ylabel("# of timestamps [-]")

        self.axes.tick_params(which="both", width=2, direction="in")
        self.axes.tick_params(which="major", length=7, direction="in")
        self.axes.tick_params(which="minor", length=4, direction="in")
        self.axes.yaxis.set_ticks_position("both")
        self.axes.xaxis.set_ticks_position("both")

        for axis in ["top", "bottom", "left", "right"]:
            self.axes.spines[axis].set_linewidth(2)

    def _plot_delta(
        self, pix1, pix2, timestamps, board_number, range_left, range_right
    ):

        files = glob("*.dat*")
        last_file = max(files, key=os.path.getctime)

        data = unpack_calib(last_file, board_number, timestamps)

        data_pair = np.vstack((data[pix1], data[pix2]))

        deltas = cd(data_pair, timestamps, range_left, range_right)

        bins = np.linspace(np.min(deltas), np.max(deltas), 100)

        self.axes.cla()
        self.axes.hist(deltas, bins=bins, color="salmon")
        self.axes.set_xlabel("\u0394t [ps]")
        self.axes.set_ylabel("# of timestamps [-]")
        self.axes.ticklabel_format(axis="y", style="sci", scilimits=(0, 0))
        self.canvas.draw()
        self.canvas.flush_events()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'LiveTimestamps_tab.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtWidgets
from app.graphic.plot_figure import PltCanvas
import glob
import os
from app.tools import timestamp_computation
import numpy as np


class Ui_LiveTimestamps(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(900, 560)
        Form.setMinimumSize(QtCore.QSize(900, 560))
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")

        self.pushButtonLoadPath = QtWidgets.QPushButton(Form)
        self.pushButtonLoadPath.setObjectName("pushButton")
        self.pushButtonLoadPath.clicked.connect(self._slot_loadpath)
        self.gridLayout.addWidget(self.pushButtonLoadPath, 0, 0, 1, 2)
        self.pathtotimestamp = ""

        self.lineEditPath = QtWidgets.QLineEdit(Form)
        self.lineEditPath.setObjectName("lineEdit")
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lineEditPath.sizePolicy().hasHeightForWidth())
        self.lineEditPath.setSizePolicy(sizePolicy)
        self.gridLayout.addWidget(self.lineEditPath, 0, 2, 1, 1)

        self.label = QtWidgets.QLabel(Form)
        self.label.setAutoFillBackground(True)
        self.label.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.label.setFrameShadow(QtWidgets.QFrame.Plain)
        self.label.setTextFormat(QtCore.Qt.PlainText)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 3, 1, 1)

        # self.widget = QtWidgets.QWidget(Form)
        self.plotWidget = PltCanvas()
        self.plotWidget.setMinimumSize(500, 400)
        self.plotWidget.setObjectName("widget")
        self.gridLayout.addWidget(self.plotWidget, 1, 0, 4, 3)

        # pixel masking scroll area
        self.scrollArea = QtWidgets.QScrollArea(Form)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 383, 346))
        self.checkBoxPixel = []
        self.scrollAreaWidgetContentslayout = QtWidgets.QGridLayout(
            self.scrollAreaWidgetContents
        )
        self.maskValidPixels = np.zeros(256)
        for col in range(4):
            for row in range(64):
                self.checkBoxPixel.append(
                    QtWidgets.QCheckBox(
                        str(row + col * 64), self.scrollAreaWidgetContents
                    )
                )
                self.scrollAreaWidgetContentslayout.addWidget(
                    self.checkBoxPixel[row + col * 64], row, col, 1, 1
                )
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 1, 3, 1, 1)
        # button for pix masking reset
        self.btnPixMaskReset = QtWidgets.QPushButton(Form)
        self.btnPixMaskReset.setObjectName("pushButton_4")
        self.gridLayout.addWidget(self.btnPixMaskReset, 2, 3, 1, 1)
        self.btnPixMaskReset.clicked.connect(self._reset_pix_mask)
        # linear scale check box
        self.checkBoxScale = QtWidgets.QCheckBox(Form)
        self.checkBoxScale.setObjectName("checkBox")
        self.gridLayout.addWidget(self.checkBoxScale, 3, 3, 1, 1)
        self.checkBoxScale.stateChanged.connect(self._slot_checkplotscale)
        # plot refresh button
        self.refreshBtn = QtWidgets.QPushButton(Form)
        self.refreshBtn.setObjectName("pushButton_2")
        self.gridLayout.addWidget(self.refreshBtn, 4, 3, 1, 1)
        self.refreshBtn.clicked.connect(self._slot_refresh)
        # real-time plotting button
        self.pushButtonStartSync = QtWidgets.QPushButton(Form)
        self.pushButtonStartSync.setMinimumSize(QtCore.QSize(385, 40))
        self.pushButtonStartSync.setObjectName("pushButton_3")
        self.gridLayout.addWidget(self.pushButtonStartSync, 5, 3, 1, 1)
        self.pushButtonStartSync.clicked.connect(self._slot_startstream)

        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 5, 0, 1, 1)

        # left xlim slider
        self.xSliderLeft = QtWidgets.QSlider(Form)
        self.xSliderLeft.setOrientation(QtCore.Qt.Horizontal)
        self.xSliderLeft.setObjectName("horizontalSlider")
        self.gridLayout.addWidget(self.xSliderLeft, 5, 1, 1, 2)
        self.xSliderLeft.valueChanged.connect(self._slot_updateLeftSlider)

        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 6, 0, 1, 1)

        # right xlim slider
        self.xSliderRight = QtWidgets.QSlider(Form)
        self.xSliderRight.setOrientation(QtCore.Qt.Horizontal)
        self.xSliderRight.setObjectName("horizontalSlider_2")
        self.gridLayout.addWidget(self.xSliderRight, 6, 1, 1, 2)
        self.xSliderRight.valueChanged.connect(self._slot_updateRightSlider)

        self.xSliderLeft.setMinimum(0)
        self.xSliderLeft.setMaximum(255)
        self.xSliderRight.setMinimum(0)
        self.xSliderRight.setMaximum(255)
        self.xSliderRight.setSliderPosition(255)
        self.leftPosition = 0
        self.rightPosition = 255

        self._retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

        # timer
        self.timer = QtCore.QTimer()
        self.timerRunning = False
        self.last_file_ctime = 0
        self.timer.timeout.connect(self._update_time_stamp)

    def _retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.pushButtonLoadPath.setText(_translate("Form", "Browse"))
        self.label.setText(_translate("Form", "Pixel Masking"))
        self.checkBoxScale.setText(_translate("Form", "Linear scale"))
        self.refreshBtn.setText(_translate("Form", "Refresh plot"))
        self.pushButtonStartSync.setText(_translate("Form", "Start stream"))
        self.label_2.setText(_translate("Form", "Left xlim"))
        self.label_3.setText(_translate("Form", "Right xlim"))
        self.btnPixMaskReset.setText(_translate("Form", "Reset pixel masking"))

    def _slot_loadpath(self):
        file = str(QtWidgets.QFileDialog.getExistingDirectory(self, "Select Directory"))
        self.lineEditPath.setText(file)
        self.pathtotimestamp = file

    def _slot_startstream(self):
        self.last_file_ctime = 0

        if self.timerRunning is True:
            self.timer.stop()
            self.timerRunning = False
            self.pushButtonStartSync.setText("Start stream")
        else:
            self.pushButtonStartSync.setText("Stop stream")
            self.timer.start(100)
            self.timerRunning = True

    def _slot_checkplotscale(self):
        if self.checkBoxScale.isChecked():
            self.plotWidget.setPlotScale(True)
        else:
            self.plotWidget.setPlotScale(False)

    def _slot_refresh(self):
        self._update_time_stamp()
        self.last_file_ctime = 0

    def _slot_updateLeftSlider(self):
        if self.xSliderLeft.value() >= self.xSliderRight.value():
            self.xSliderLeft.setValue(self.xSliderRight.value() - 1)
        self.leftPosition = self.xSliderLeft.value()

    def _slot_updateRightSlider(self):
        if self.xSliderRight.value() <= self.xSliderLeft.value():
            self.xSliderRight.setValue(self.xSliderLeft.value() + 1)
        self.rightPosition = self.xSliderRight.value()

    def _update_time_stamp(self):
        self._mask_pixels()
        os.chdir(self.pathtotimestamp)
        DATA_FILES = glob.glob("*.dat*")
        print("_update_time_stamp: timer running")
        try:
            last_file = max(DATA_FILES, key=os.path.getctime)
            new_file_ctime = os.path.getctime(last_file)

            if new_file_ctime > self.last_file_ctime:

                self.last_file_ctime = new_file_ctime

                validtimestemps, peak = timestamp_computation.get_nmr_validtimestamps(
                    self.pathtotimestamp + "/" + last_file, np.arange(145, 155, 1), 512
                )
                validtimestemps = validtimestemps * self.maskValidPixels
                self.plotWidget.setPlotData(
                    np.arange(0, 256, 1),
                    validtimestemps,
                    peak,
                    [self.leftPosition, self.rightPosition],
                )
        except ValueError:
            print("_update_time_stamp:  waiting for a file")

    def _mask_pixels(self):
        '''
        Function for masking the chosen pixels.

        Returns
        -------
        None.

        '''
        for i in range(256):
            if self.checkBoxPixel[i].isChecked():
                self.maskValidPixels[i] = 0
            else:
                self.maskValidPixels[i] = 1

    def _reset_pix_mask(self):
        """
        Function for resetting the pixel masking by unchecking all pixel
        mask check boxes.

        Returns
        -------
        None.

        """
        for i in range(256):
            cb = self.scrollAreaWidgetContentslayout.itemAt(i).widget()
            cb.setChecked(False)



"""Module for plotting photon count from two chosen pixel in real time.

Unpacks the binary data, finds the two pixels requested and plots the
number of photons registered by the two pixels. x-axis limits are given 
by the slider, upper y-axis limit is set as maximum of the two counts
plus 10 percent.

"""

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.backends.backend_qt5agg import (
    FigureCanvasQTAgg as FigureCanvas,
)
from matplotlib.backends.backend_qt5agg import (
    NavigationToolbar2QT as NavigationToolbar,
)
from matplotlib.figure import Figure
from PyQt5.QtWidgets import QVBoxLayout, QWidget


class PltCanvas_MZI(QWidget):
    def __init__(self, parent=None, width=7, height=4, dpi=100):
        """Creation of the figure widget.

        The widget is created with the bar with options.

        Parameters
        ----------
        width : int, optional
            Figure widget width, by default 7.
        height : int, optional
            Figure widget height, by default 4.
        dpi : int, optional
            Figure widget dpi, by default 100.
        """
        super(PltCanvas_MZI, self).__init__(parent)

        # For 'dark_background' style
        plt.style.use("dark_background")

        # a figure instance to plot on
        self.figure = Figure(figsize=(width, height), dpi=100)
        self.canvas = FigureCanvas(self.figure)

        self.toolbar = NavigationToolbar(self.canvas, self)
        self.ax = self.figure.add_subplot(111)
        self.ax2 = self.ax.twinx()
        self.figure.subplots_adjust(
            left=0.15, right=0.9, top=0.945, bottom=0.12
        )

        # creating a Vertical Box layout
        self.layout = QVBoxLayout(self)
        self.layout.addWidget(self.canvas)
        self.layout.addWidget(self.toolbar)

        self.setLayout(self.layout)

        self.setplotparameters()

        # Upper ylim for the plot
        self.upper_ylim = 0

    def setplotparameters(self, fontsize: int = 16):
        """Figure parameters manipulation.

        Set font size, axes labels. Set the width and orientation of the
        axes ticks.

        """
        plt.rcParams.update({"font.size": fontsize})
        self.ax.set_xlabel("Pixel (-)", fontsize=fontsize)
        self.ax.set_ylabel("# of timestamps (-)", fontsize=fontsize)

        self.ax.tick_params(which="both", width=2, direction="in")
        self.ax.tick_params(
            which="major", length=7, direction="in", labelsize=fontsize
        )
        self.ax.tick_params(
            which="minor", length=4, direction="in", labelsize=fontsize
        )
        self.ax2.tick_params(
            which="major", length=7, direction="in", labelsize=fontsize
        )
        self.ax2.tick_params(
            which="minor", length=4, direction="in", labelsize=fontsize
        )
        self.ax.yaxis.set_ticks_position("both")
        self.ax.xaxis.set_ticks_position("both")

        for axis in ["top", "bottom", "left", "right"]:
            self.ax.spines[axis].set_linewidth(2)

    def setPlotData_MZI(self, xdataplot, yplotdata1, yplotdata2, xLim, yLim):
        """Plot data.

        Plot the provided data while following the state of the axis
        limits and the switch for plotting vertical lines at positions
        64, 128, and 192.

        Parameters
        ----------
        xdataplot : array
            Data for the x-axis: pixel numbers.
        yplotdata : array-like
            Data for the y-axis: number of timestamps.
        xLim : list
            Limits for the x-axis.
        grouping : bool, optional
            Switch for plotting vertical lines at positiong 64, 128, and
            192, by default False.
        """
        # self.ax.cla()
        self.ax.plot(xdataplot, yplotdata1, "-o", color="indianred")
        self.ax2.plot(xdataplot, yplotdata2, "-o", color="teal")
        self.ax.relim()
        self.ax.autoscale_view()
        self.setplotparameters()
        self.ax.set_xlim(xLim, xdataplot[-1] + 2)

        new_ylim = np.max([yplotdata1[-1], yplotdata2[-1]]) * 1.1
        if new_ylim > self.upper_ylim:
            self.upper_ylim = new_ylim

        self.ax.set_ylim(yLim, self.upper_ylim)
        self.ax2.set_ylim(yLim, self.upper_ylim)
        self.ax.ticklabel_format(axis="y", style="sci", scilimits=(0, 0))
        self.ax2.ticklabel_format(axis="y", style="sci", scilimits=(0, 0))
        self.figure.canvas.draw()
        self.figure.canvas.flush_events()

    def setPlotScale(self, scaleLin):
        """Switches plot scale between logarithmic and linear."""
        if scaleLin:
            self.ax.set_yscale("linear")
            self.canvas.draw()
            self.canvas.flush_events()
        else:
            self.ax.set_yscale("log")
            self.canvas.draw()
            self.canvas.flush_events()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'SinglePixelHistogram.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets
import os
from app.graphic.single_pixel_histogram import SinglePixelHistogram


class Ui_SinglePixelHistogram(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(900, 560)
        Form.setMinimumSize(QtCore.QSize(900, 560))

        # pixel number
        self.pix = None
        # path
        self.folder = ""

        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setObjectName("gridLayout")
        self.btnBrowse = QtWidgets.QPushButton(Form)
        self.btnBrowse.setObjectName("pushButton")
        self.gridLayout.addWidget(self.btnBrowse, 0, 0, 1, 1)

        self.btnBrowse.clicked.connect(self._get_dir)

        self.path = QtWidgets.QLineEdit(Form)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.path.sizePolicy().hasHeightForWidth())
        self.path.setSizePolicy(sizePolicy)
        self.path.setObjectName("lineEdit")
        self.gridLayout.addWidget(self.path, 0, 1, 1, 1)

        spacerItem = QtWidgets.QSpacerItem(
            382, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum
        )
        self.gridLayout.addItem(spacerItem, 0, 2, 1, 2)

        # self.widget = QtWidgets.QWidget(Form)
        self.figureWidget = SinglePixelHistogram()
        self.figureWidget.setMinimumSize(QtCore.QSize(486, 445))
        self.figureWidget.setObjectName("widget")
        self.gridLayout.addWidget(self.figureWidget, 1, 0, 2, 2)

        self.label = QtWidgets.QLabel(Form)
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label.setFont(font)
        self.label.setTextFormat(QtCore.Qt.AutoText)
        self.label.setScaledContents(False)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 2, 1, 1)

        self.pixInput = QtWidgets.QLineEdit(Form)
        self.pixInput.setMinimumSize(QtCore.QSize(0, 28))
        self.pixInput.setObjectName("pixInput")
        self.pixInput.setValidator(QtGui.QIntValidator())
        self.pixInput.setMaxLength(4)
        self.pixInput.setAlignment(QtCore.Qt.AlignCenter)
        self.pixInput.setFont(QtGui.QFont("Arial", 20))
        self.gridLayout.addWidget(self.pixInput, 1, 3, 1, 1)

        self.pixInput.returnPressed.connect(lambda: self._pix_input())

        self.btnRfrshPlot = QtWidgets.QPushButton(Form)
        self.btnRfrshPlot.setObjectName("pushButton_2")
        self.gridLayout.addWidget(self.btnRfrshPlot, 2, 2, 1, 2)
        
        self.btnRfrshPlot.clicked.connect(self._refresh_plot)

        self._retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def _retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.btnBrowse.setText(_translate("Form", "Browse"))
        self.label.setText(_translate("Form", "Enter pixel index:"))
        self.btnRfrshPlot.setText(_translate("Form", "Refresh plot"))

    def _get_dir(self):
        self.folder = str(
            QtWidgets.QFileDialog.getExistingDirectory(self, "Select Directory")
        )
        self.path.setText(self.folder)

    def _pix_input(self):

        self.pix = int(self.pixInput.text())
        os.chdir(self.folder)

        self.figureWidget._plot_hist(self.pix)

    def _refresh_plot(self):
        
        self.pix = int(self.pixInput.text())
        os.chdir(self.folder)

        self.figureWidget._plot_hist(self.pix)



"""Module for plotting data in the real-time plotting tab.

Plots the data provided as number of timestamps vs. pixel number. Options
for changing the plot scale (linear or logarithmic) and plotting of 
vertical lines at positions 64, 128, and 192 are provided. The figure
widget is generated with the matplotlib navigation bar for additional
control over the plot.

"""

import matplotlib.pyplot as plt
from matplotlib.backends.backend_qt5agg import (
    FigureCanvasQTAgg as FigureCanvas,
)
from matplotlib.backends.backend_qt5agg import (
    NavigationToolbar2QT as NavigationToolbar,
)
from matplotlib.figure import Figure
from PyQt5.QtWidgets import QVBoxLayout, QWidget


class PltCanvas(QWidget):
    def __init__(self, parent=None, width=7, height=4, dpi=100):
        """Creation of the figure widget.

        The widget is created with the bar with options.

        Parameters
        ----------
        width : int, optional
            Figure widget width, by default 7.
        height : int, optional
            Figure widget height, by default 4.
        dpi : int, optional
            Figure widget dpi, by default 100.
        """
        super(PltCanvas, self).__init__(parent)

        # For 'dark_background' style
        plt.style.use("dark_background")

        # a figure instance to plot on
        self.figure = Figure(figsize=(width, height), dpi=100)
        self.canvas = FigureCanvas(self.figure)

        self.toolbar = NavigationToolbar(self.canvas, self)
        self.ax = self.figure.add_subplot(111)
        self.figure.subplots_adjust(
            left=0.15, right=0.97, top=0.945, bottom=0.12
        )

        # creating a Vertical Box layout
        self.layout = QVBoxLayout(self)
        self.layout.addWidget(self.canvas)
        self.layout.addWidget(self.toolbar)

        self.setLayout(self.layout)

        self.setplotparameters()

    def setplotparameters(self, fontsize: int = 16):
        """Figure parameters manipulation.

        Set font size, axes labels. Set the width and orientation of the
        axes ticks.

        """
        plt.rcParams.update({"font.size": fontsize})
        self.ax.set_xlabel("Pixel (-)", fontsize=fontsize)
        self.ax.set_ylabel("# of timestamps (-)", fontsize=fontsize)

        self.ax.tick_params(which="both", width=2, direction="in")
        self.ax.tick_params(
            which="major", length=7, direction="in", labelsize=fontsize
        )
        self.ax.tick_params(
            which="minor", length=4, direction="in", labelsize=fontsize
        )
        self.ax.yaxis.set_ticks_position("both")
        self.ax.xaxis.set_ticks_position("both")

        for axis in ["top", "bottom", "left", "right"]:
            self.ax.spines[axis].set_linewidth(2)

    def setPlotData(self, xdataplot, yplotdata, xLim, grouping: bool = False):
        """Plot data.

        Plot the provided data while following the state of the axis
        limits and the switch for plotting vertical lines at positions
        64, 128, and 192.

        Parameters
        ----------
        xdataplot : array
            Data for the x-axis: pixel numbers.
        yplotdata : array-like
            Data for the y-axis: number of timestamps.
        xLim : list
            Limits for the x-axis.
        grouping : bool, optional
            Switch for plotting vertical lines at positiong 64, 128, and
            192, by default False.
        """
        self.ax.cla()
        self.ax.plot(yplotdata, "-o", color="indianred")
        if grouping is True:
            self.ax.vlines(
                x=(64, 128, 192), ymin=0, ymax=yplotdata.max(), color="teal"
            )
        self.ax.relim()
        self.ax.autoscale_view()
        self.setplotparameters()
        self.ax.set_xlim(xLim[0], xLim[1])
        self.ax.ticklabel_format(axis="y", style="sci", scilimits=(0, 0))
        self.figure.canvas.draw()
        self.figure.canvas.flush_events()

    def setPlotScale(self, scaleLin):
        """Switches plot scale between logarithmic and linear."""
        if scaleLin:
            self.ax.set_yscale("linear")
            self.canvas.draw()
            self.canvas.flush_events()
        else:
            self.ax.set_yscale("log")
            self.canvas.draw()
            self.canvas.flush_events()



"""This script generates the tab for online sensor population plotting.

The tab can be used to introduce changes to the setup while following
the changes in real-time (depending on the actual data file size, the
plotting can take minutes).

"""

import glob
import os

import numpy as np
from daplis_rtp.functions.sen_pop import sen_pop
from daplis_rtp.gui.plot_figure import PltCanvas
from daplis_rtp.gui.ui.LiveTimestamps_tab_c import Ui_Form
from PyQt5 import QtCore, QtWidgets, uic


class LiveTimestamps(QtWidgets.QWidget):
    def __init__(self, parent=None):
        """Creation of the tab.

        The tab is generated with a 'Browse' button along with a line
        edit field for choosing/inserting the address of the data
        file to plot. Combo boxes for LinoSPAD2 daughterboard number and
        the firmware version are generated. A check box for applying and
        a button for undoing the mask are generated. A spin box for
        the number of timestamps per pixel/TDC per cycle is provided. A
        widget with a grid of 4x64 of checkboxes with pixel numbers
        for masking single pixels is generated. A check box for switching
        between a linear and a logarithmic scale of the plot along with
        a check box for plotting vertical lines at positions 64, 128, and
        192 are provided (the latter can be used for firmware versions
        2208 and 2212s for setup alignment). Buttons 'Refresh plot' for
        refreshing the plot and 'Start stream' for plotting the
        last file found are created. Two sliders for the left and right
        limits for the x-axis are created.

        """
        super().__init__(parent)
        # os.chdir(r"C:\Users\bruce\Documents\Python Scripts\daplis-rtp\src\daplis_rtp\gui\ui")
        # uic.loadUi(
        #     r"LiveTimestamps_tab_c.ui",
        #     self,
        # )
        # os.chdir("../..")

        self.ui = Ui_Form()
        self.ui.setupUi(self)

        # Dynamically bind all child widgets to `self`
        for attr_name in dir(self.ui):
            if not attr_name.startswith("__"):  # Skip dunder methods
                attr = getattr(self.ui, attr_name)
                setattr(self, attr_name, attr)

        self.show()

        self.pathtotimestamp = ""

        # Browse button
        self.pushButton_browse.clicked.connect(self.get_dir)

        # Scroll area with check boxes
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 383, 346))
        self.checkBoxPixel = []
        self.scrollAreaWidgetContentslayout = QtWidgets.QGridLayout(
            self.scrollAreaWidgetContents
        )
        self.maskValidPixels = np.zeros(256)
        for col in range(4):
            for row in range(64):
                self.checkBoxPixel.append(
                    QtWidgets.QCheckBox(
                        str(row + col * 64), self.scrollAreaWidgetContents
                    )
                )
                self.scrollAreaWidgetContentslayout.addWidget(
                    self.checkBoxPixel[row + col * 64], row, col, 1, 1
                )
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        # Figure widget
        self.widget_figure = PltCanvas()
        # self.widget_figure.setMinimumSize(500, 400)
        # self.widget_figure.setFixedSize(500, 425)
        self.widget_figure.setObjectName("widget")
        self.gridLayout.addWidget(self.widget_figure, 1, 0, 4, 3)

        # Sliders
        self.horizontalSlider_leftXLim.valueChanged.connect(
            self.slot_updateLeftSlider
        )
        self.horizontalSlider_rightXLim.valueChanged.connect(
            self.slot_updateRightSlider
        )

        self.horizontalSlider_leftXLim.setMinimum(0)
        self.horizontalSlider_leftXLim.setMaximum(255)
        self.horizontalSlider_rightXLim.setMinimum(0)
        self.horizontalSlider_rightXLim.setMaximum(255)
        self.horizontalSlider_rightXLim.setSliderPosition(255)
        self.leftPosition = 0
        self.rightPosition = 255

        # Pixel masking
        self.checkBox_presetMask_2.stateChanged.connect(self.presetmask_pixels)

        self.checkBox_linearScale_2.stateChanged.connect(
            self.slot_checkplotscale_2
        )

        self.pushButton_resetMask_2.clicked.connect(self.reset_pix_mask)

        self.path_to_main = os.getcwd()

        self.comboBox_mask_2.activated.connect(self.reset_pix_mask)

        # Refresh plot and start stream buttons
        self.pushButton_refreshPlot.clicked.connect(self.slot_refresh)

        self.pushButton_startStream.clicked.connect(self.slot_startstream)

        # Check box for plotting 3 vertical lines at position x=64,
        # 128, 192 (FW 2208)
        self.grouping = False
        self.checkBox_grouping_2.stateChanged.connect(
            self.slot_checkBox_grouping_2
        )

        # Set directory if path was pasted instead of chosen with the
        # 'Browse' button
        self.lineEdit_browse.textChanged.connect(self.change_path)

        # Timer preset
        self.timer = QtCore.QTimer()
        self.timerRunning = False
        self.last_file_ctime = 0
        self.timer.timeout.connect(self.update_time_stamp)

    # Testing adaptive fontsize
    def resizeEvent(self, event):

        # Define minimum and maximum width and corresponding font sizes
        min_width = 908
        max_width = 3810
        min_fontsize = 16
        max_fontsize = 40

        # Get the current window width
        current_width = self.size().width()

        # Ensure the current width stays within the bounds
        current_width = max(min_width, min(max_width, current_width))

        # Calculate the new font size using linear interpolation
        new_font_size = min_fontsize + (
            (current_width - min_width) / (max_width - min_width)
        ) * (max_fontsize - min_fontsize)

        self.widget_figure.setplotparameters(fontsize=new_font_size)

        super().resizeEvent(event)

    def get_dir(self):
        """Called when the 'browse' button is pressed.

        Sets the path variable to the address chosen.

        """
        file = str(
            QtWidgets.QFileDialog.getExistingDirectory(
                self, "Select Directory"
            )
        )
        self.lineEdit_browse.setText(file)
        self.pathtotimestamp = file

    def change_path(self):
        """Called when address is inserted to the line edit.

        Sets the path variable to the address inserted.

        """
        self.pathtotimestamp = self.lineEdit_browse.text()

    def slot_startstream(self):
        """Called when the 'Start stream' button is pressed.

        Starts an infinite cycle of refreshing the plot when new files
        are found in the folder.

        """
        self.last_file_ctime = 0

        if self.timerRunning is True:
            self.timer.stop()
            self.timerRunning = False
            self.pushButton_startStream.setText("Start stream")
        else:
            self.pushButton_startStream.setText("Stop stream")
            self.timer.start(100)
            self.timerRunning = True

    def slot_stopstream(self):
        self.timer.stop()
        self.timerRunning = False
        self.pushButton_startStream.setText("Start stream")
        self.last_file_ctime = 0

    def slot_checkplotscale_2(self):
        """Called when state of the check box for scale is changed.

        Switches between logarithmic and linear scale of the plot.

        """
        if self.checkBox_linearScale_2.isChecked():
            self.widget_figure.setPlotScale(True)
        else:
            self.widget_figure.setPlotScale(False)

    def slot_refresh(self):
        """Called when the 'Refresh button' is pressed.

        Refreshes the plot, either the current one or updates with the
        new data if new data were taken.

        """
        self.update_time_stamp()
        self.last_file_ctime = 0

    def slot_updateLeftSlider(self):
        """Called when left slider state has changed.

        Updates the left x-axis limit based on the position of the
        slider.

        """
        if (
            self.horizontalSlider_leftXLim.value()
            >= self.horizontalSlider_rightXLim.value()
        ):
            self.horizontalSlider_leftXLim.setValue(
                self.horizontalSlider_rightXLim.value() - 1
            )
        self.leftPosition = self.horizontalSlider_leftXLim.value()

    def slot_updateRightSlider(self):
        """Called when right slider state has changed.

        Updates the right x-axis limit based on the position of the
        slider.

        """
        if (
            self.horizontalSlider_rightXLim.value()
            <= self.horizontalSlider_leftXLim.value()
        ):
            self.horizontalSlider_rightXLim.setValue(
                self.horizontalSlider_leftXLim.value() + 1
            )
        self.rightPosition = self.horizontalSlider_rightXLim.value()

    def update_time_stamp(self):
        """Called during the cycle of real-time plotting.

        Load data from the last data file found in the directory
        provided.

        """
        stopping = False
        self.mask_pixels()
        os.chdir(self.pathtotimestamp)
        DATA_FILES = glob.glob("*.dat*")
        try:
            last_file = max(DATA_FILES, key=os.path.getctime)
            new_file_ctime = os.path.getctime(last_file)
        except ValueError:
            msg_window = QtWidgets.QMessageBox()
            msg_window.setText(
                "No data files found, check the working directory."
            )
            msg_window.setWindowTitle("Error")
            msg_window.exec_()
            self.slot_stopstream()
            stopping = True
        if stopping is False:
            try:
                if new_file_ctime > self.last_file_ctime:
                    self.last_file_ctime = new_file_ctime

                    validtimestamps = sen_pop(
                        self.pathtotimestamp + "/" + last_file,
                        board_number=self.comboBox_mask_2.currentText(),
                        fw_ver=self.comboBox_FW_2.currentText(),
                        timestamps=self.spinBox_timestamps_2.value(),
                        pix_add_fix=self.checkBox_pix_add_fix.isChecked(),
                    )
                    validtimestamps = validtimestamps * self.maskValidPixels
                    self.widget_figure.setPlotData(
                        np.arange(0, 256, 1),
                        validtimestamps,
                        [self.leftPosition, self.rightPosition],
                        self.grouping,
                    )

                    copy_for_max = np.sort(np.copy(validtimestamps))

                    self.lcdTopMostPixel1.display(
                        np.argwhere(validtimestamps == copy_for_max[-1])[0][0]
                    )
                    self.lcdTopMostPixel2.display(
                        np.argwhere(validtimestamps == copy_for_max[-2])[0][0]
                    )

            except ValueError:
                msg_window = QtWidgets.QMessageBox()
                msg_window.setText(
                    "Cannot unpack data, check the timestamp setting and "
                    "the firmware version."
                )
                msg_window.setWindowTitle("Error")
                msg_window.exec_()
                self.slot_stopstream()
                stopping = True

    def mask_pixels(self):
        """
        Function for masking the chosen pixels.

        """
        for i in range(256):
            if self.checkBoxPixel[i].isChecked():
                self.maskValidPixels[i] = 0
            else:
                self.maskValidPixels[i] = 1

    def presetmask_pixels(self):
        """Called when the check box 'Preset mask' is checked.

        Uses the masking data provided in the 'params' folder for
        masking of the warm/hot pixels. Uses the LinoSPAD2 daughterboard
        number to load appropriate data.

        """

        if self.checkBox_presetMask_2.isChecked():
            try:
                if os.getcwd() != self.path_to_main + "/params/masks":
                    os.chdir(self.path_to_main + "/params/masks")
                file = glob.glob(
                    "*{}_{}*".format(
                        self.comboBox_mask_2.currentText(),
                        self.comboBox_mb_2.currentText(),
                    )
                )[0]
                mask = np.genfromtxt(file, delimiter=",", dtype="int")
                for i in mask:
                    self.maskValidPixels[i] = 0
                    cb = self.scrollAreaWidgetContentslayout.itemAt(i).widget()
                    cb.setChecked(True)
            except IndexError:
                self.checkBox_presetMask_2.setCheckState(0)
                msg_window = QtWidgets.QMessageBox()
                msg_window.setText(
                    "No mask data were found for the given daughterboard, "
                    "motherboard, and firmware combination."
                )
                msg_window.setWindowTitle("Error")
                msg_window.exec_()

        else:
            for i in range(256):
                # self.maskValidPixels[i] = 0
                cb = self.scrollAreaWidgetContentslayout.itemAt(i).widget()
                cb.setChecked(False)

    def reset_pix_mask(self):
        """
        Function for resetting the pixel masking by unchecking all pixel
        mask checkboxes.

        """
        for i in range(256):
            cb = self.scrollAreaWidgetContentslayout.itemAt(i).widget()
            cb.setChecked(False)
        self.checkBox_presetMask_2.setChecked(False)

    def slot_checkBox_grouping_2(self):
        """Called when check box for plotting lines state changed."""
        if self.checkBox_grouping_2.isChecked():
            self.grouping = True
        else:
            self.grouping = False



""" Function for calculating timestamp differences for the given pair of
pixels.

This file can also be imported as a module and contains the following
functions:

    * calc_diff_2208 - calculate timestamp differences for a given
    pair of pixels. Returns an array of differences.
    
    * calc_diff_2212

"""

import numpy as np

def calc_diff(
    data_pair,
    timestamps: int = 512,
    range_left: float = -2.5e3,
    range_right: float = 2.5e3,
):
    """
    Function for calculating timestamp differences for a given pair of pixels.

    Parameters
    ----------
    data_pair : array-like
        Data from 2 pixels.
    timestamps : int, optional
        Number of timestamps per pixel per acquisition window. The default is
        512.
    range_left : float, optional
        Left limit for the timestamp differences. Values below that are not
        taken into account and the differences are not returned. The default
        is -2.5e3.
    range_right : float, optional
        Right limit for the timestamp differences. Values above that are not
        taken into account and the differences are not returned. The default
        is 2.5e3.

    Returns
    -------
    output : array-like
        An array of timestamp differences for the given pair of pixels.

    """

    timestamps_total = len(data_pair[0])

    output = []

    acq = 0  # number of acq cycle
    for j in range(timestamps_total):
        if j % timestamps == 0:
            acq = acq + 1  # next acq cycle
        if data_pair[0][j] == -1:
            continue
        for p in range(timestamps):
            n = timestamps * (acq - 1) + p
            if data_pair[1][n] == -1:
                continue
            else:
                delta = data_pair[0][j] - data_pair[1][n]
                if delta > range_right:
                    continue
                elif delta < range_left:
                    continue
                else:
                    output.append(delta)
    return output


def calc_diff_2212(data1, data2, cycle_ends, delta_window: float = 10e3):
    """Calculate timestamp differences for firmware 2212.

    Calculate timestamp differences for two given pixels and LinoSPAD2
    firmware version 2212b.

    Parameters
    ----------
    data1 : array-like
        Array of data from the first pixel of the pair for which
        timestamp differences are calculated.
    data2 : array-like
        Array of data from the second pixel of the pair for which
        timestamp differecnes are calculated.
    cycle_ends : array-like
        Array of positions of '-2's that indicate ends of cycles.
    delta_window : float, optional
        A width of a window in which the number of timestamp differences
        are counted. The default value is 10 ns. The default is 10e3.

    Returns
    -------
    deltas_out : list
        All timestamp differences found in a given time window for a
        given pair of pixels.

    """
    deltas_out = []

    for cyc in range(len(cycle_ends) - 1):
        data1_cycle = data1[cycle_ends[cyc] : cycle_ends[cyc + 1]]
        data2_cycle = data2[cycle_ends[cyc] : cycle_ends[cyc + 1]]

        # calculate delta t
        tmsp1 = data1_cycle[np.where(data1_cycle > 0)[0]]
        tmsp2 = data2_cycle[np.where(data2_cycle > 0)[0]]
        for t1 in tmsp1:
            # calculate all timestamp differences in the given cycle
            deltas = tmsp2 - t1
            # take values in the given delta t window only
            ind = np.where(np.abs(deltas) < delta_window)[0]
            deltas_out.extend(deltas[ind])

    return deltas_out


"""Application starter.

Run this file in the terminal to start the application.

"""

import os
import sys

import qdarkstyle
from PyQt5.QtWidgets import QApplication, QMainWindow

from daplis_rtp.gui.ui.mainwindow import Ui_MainWindow


class MainWindow(QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setupUi(self)

    # for stopping the script upon closing the app window
    def closeEvent(self, event):
        QApplication.quit()


def main():
    app = QApplication(sys.argv)
    window = MainWindow()
    # For dark theme
    app.setStyleSheet(qdarkstyle.load_stylesheet())
    window.show()
    app.exec()


if __name__ == "__main__":
    main()



"""This module contains old functions.

* get_nmr_validtimestamps

"""

import numpy as np
import sys

sys.path.append("..")
from tools.unpack import unpack_bin
import sys


def get_nmr_validtimestamps(path, board_number, fw_ver, timestamps: int = 512):
    if fw_ver == "2208":
        data = unpk.unpack_calib(path, board_number, timestamps)

        valid_per_pixel = np.zeros(256)

        for j in range(len(data)):
            valid_per_pixel[j] = len(np.where(data[j] > 0)[0])
    elif fw_ver == "2212b":
        data = unpk.unpack_2212(path, board_number, fw_ver, timestamps)

        valid_per_pixel = np.zeros(256)

        for j in range(len(data)):
            valid_per_pixel[j] = len(np.where(data["{}".format(j)] > 0)[0])

    return valid_per_pixel
